﻿Namespace ViewModel
    Public Class FuncionariosViewModel
        Inherits BaseNotifyPropertyChanged

        Public Property Funcionarios As System.Collections.ObjectModel.ObservableCollection(Of Model.Funcionario)

        Private _funcionarioSelecionado As Model.Funcionario
        Public Property FuncionarioSelecionado() As Model.Funcionario
            Get
                Return _funcionarioSelecionado
            End Get
            Set
                SetField(_funcionarioSelecionado, Value)
                Deletar.RaiseCanExecuteChanged()
                Editar.RaiseCanExecuteChanged()
            End Set
        End Property

        Public Sub New()
            Funcionarios = New System.Collections.ObjectModel.ObservableCollection(Of Model.Funcionario)()
            Funcionarios.Add(New Model.Funcionario() With {
                .Id = 1,
                .Nome = "André",
                .Sobrenome = "Lima",
                .DataNascimento = New DateTime(1984, 12, 31),
                .Sexo = Model.Sexo.Masculino,
                .EstadoCivil = Model.EstadoCivil.Casado,
                .DataAdmissao = New DateTime(2010, 1, 1)})

            FuncionarioSelecionado = Funcionarios.FirstOrDefault()
        End Sub

#Region "Commands"
#Region "Deletar"
        Public Property Deletar As New DeletarCommand

        Public Class DeletarCommand
            Inherits BaseCommand
            Public Overrides Function CanExecute(parameter As Object) As Boolean
                Dim viewModel = TryCast(parameter, FuncionariosViewModel)
                Return viewModel IsNot Nothing AndAlso viewModel.FuncionarioSelecionado IsNot Nothing
            End Function

            Public Overrides Sub Execute(parameter As Object)
                Dim viewModel = DirectCast(parameter, FuncionariosViewModel)
                viewModel.Funcionarios.Remove(viewModel.FuncionarioSelecionado)
                viewModel.FuncionarioSelecionado = viewModel.Funcionarios.FirstOrDefault()
            End Sub
        End Class
#End Region

#Region "Novo"
        Public Property Novo As New NovoCommand

        Public Class NovoCommand
            Inherits BaseCommand
            Public Overrides Function CanExecute(parameter As Object) As Boolean
                Return TypeOf parameter Is FuncionariosViewModel
            End Function

            Public Overrides Sub Execute(parameter As Object)
                Dim viewModel = DirectCast(parameter, FuncionariosViewModel)
                Dim funcionario = New Model.Funcionario()
                Dim maxId = 0
                If viewModel.Funcionarios.Any() Then
                    maxId = viewModel.Funcionarios.Max(Function(f) f.Id)
                End If
                funcionario.Id = maxId + 1

                Dim fw = New FuncionarioWindow()
                fw.DataContext = funcionario
                fw.ShowDialog()

                If fw.DialogResult.HasValue AndAlso fw.DialogResult.Value Then
                    viewModel.Funcionarios.Add(funcionario)
                    viewModel.FuncionarioSelecionado = funcionario
                End If
            End Sub
        End Class
#End Region

#Region "Editar"
        Public Property Editar As New EditarCommand

        Public Class EditarCommand
            Inherits BaseCommand
            Public Overrides Function CanExecute(parameter As Object) As Boolean
                Dim viewModel = TryCast(parameter, FuncionariosViewModel)
                Return viewModel IsNot Nothing AndAlso viewModel.FuncionarioSelecionado IsNot Nothing
            End Function

            Public Overrides Sub Execute(parameter As Object)
                Dim viewModel = DirectCast(parameter, FuncionariosViewModel)
                Dim cloneFuncionario = DirectCast(viewModel.FuncionarioSelecionado.Clone(), Model.Funcionario)
                Dim fw = New FuncionarioWindow()
                fw.DataContext = cloneFuncionario
                fw.ShowDialog()

                If fw.DialogResult.HasValue AndAlso fw.DialogResult.Value Then
                    viewModel.FuncionarioSelecionado.Nome = cloneFuncionario.Nome
                    viewModel.FuncionarioSelecionado.Sobrenome = cloneFuncionario.Sobrenome
                    viewModel.FuncionarioSelecionado.DataNascimento = cloneFuncionario.DataNascimento
                    viewModel.FuncionarioSelecionado.Sexo = cloneFuncionario.Sexo
                    viewModel.FuncionarioSelecionado.EstadoCivil = cloneFuncionario.EstadoCivil
                    viewModel.FuncionarioSelecionado.DataAdmissao = cloneFuncionario.DataAdmissao
                End If
            End Sub
        End Class
#End Region
#End Region
    End Class
End Namespace