﻿using System;
using System.Linq;

namespace WPFMVVMCRUD.ViewModel
{
    public class FuncionariosViewModel : BaseNotifyPropertyChanged
    {
        public System.Collections.ObjectModel.ObservableCollection<Model.Funcionario> Funcionarios { get; private set; }

        private Model.Funcionario _funcionarioSelecionado;
        public Model.Funcionario FuncionarioSelecionado
        {
            get { return _funcionarioSelecionado; }
            set
            {
                SetField(ref _funcionarioSelecionado, value);
                Deletar.RaiseCanExecuteChanged();
                Editar.RaiseCanExecuteChanged();
            }
        }

        public FuncionariosViewModel()
        {
            Funcionarios = new System.Collections.ObjectModel.ObservableCollection<Model.Funcionario>();
            Funcionarios.Add(new Model.Funcionario()
            {
                Id = 1,
                Nome = "André",
                Sobrenome = "Lima",
                DataNascimento = new DateTime(1984, 12, 31),
                Sexo = Model.Sexo.Masculino,
                EstadoCivil = Model.EstadoCivil.Casado,
                DataAdmissao = new DateTime(2010, 1, 1)
            });

            FuncionarioSelecionado = Funcionarios.FirstOrDefault();
        }

        #region Commands
        #region Deletar
        public DeletarCommand Deletar { get; private set; } = new DeletarCommand();

        public class DeletarCommand : BaseCommand
        {
            public override bool CanExecute(object parameter)
            {
                var viewModel = parameter as FuncionariosViewModel;
                return viewModel != null && viewModel.FuncionarioSelecionado != null;
            }

            public override void Execute(object parameter)
            {
                var viewModel = (FuncionariosViewModel)parameter;
                viewModel.Funcionarios.Remove(viewModel.FuncionarioSelecionado);
                viewModel.FuncionarioSelecionado = viewModel.Funcionarios.FirstOrDefault();
            }
        }
        #endregion

        #region Novo
        public NovoCommand Novo { get; private set; } = new NovoCommand();

        public class NovoCommand : BaseCommand
        {
            public override bool CanExecute(object parameter)
            {
                return parameter is FuncionariosViewModel;
            }

            public override void Execute(object parameter)
            {
                var viewModel = (FuncionariosViewModel)parameter;
                var funcionario = new Model.Funcionario();
                var maxId = 0;
                if (viewModel.Funcionarios.Any())
                {
                    maxId = viewModel.Funcionarios.Max(f => f.Id);
                }
                funcionario.Id = maxId + 1;

                var fw = new FuncionarioWindow();
                fw.DataContext = funcionario;
                fw.ShowDialog();
                
                if (fw.DialogResult.HasValue && fw.DialogResult.Value)
                {
                    viewModel.Funcionarios.Add(funcionario);
                    viewModel.FuncionarioSelecionado = funcionario;
                }
            }
        }
        #endregion

        #region Editar
        public EditarCommand Editar { get; private set; } = new EditarCommand();

        public class EditarCommand : BaseCommand
        {
            public override bool CanExecute(object parameter)
            {
                var viewModel = parameter as FuncionariosViewModel;
                return viewModel != null && viewModel.FuncionarioSelecionado != null;
            }

            public override void Execute(object parameter)
            {
                var viewModel = (FuncionariosViewModel)parameter;
                var cloneFuncionario = (Model.Funcionario)viewModel.FuncionarioSelecionado.Clone();
                var fw = new FuncionarioWindow();
                fw.DataContext = cloneFuncionario;
                fw.ShowDialog();

                if (fw.DialogResult.HasValue && fw.DialogResult.Value)
                {
                    viewModel.FuncionarioSelecionado.Nome = cloneFuncionario.Nome;
                    viewModel.FuncionarioSelecionado.Sobrenome = cloneFuncionario.Sobrenome;
                    viewModel.FuncionarioSelecionado.DataNascimento = cloneFuncionario.DataNascimento;
                    viewModel.FuncionarioSelecionado.Sexo = cloneFuncionario.Sexo;
                    viewModel.FuncionarioSelecionado.EstadoCivil = cloneFuncionario.EstadoCivil;
                    viewModel.FuncionarioSelecionado.DataAdmissao = cloneFuncionario.DataAdmissao;
                }
            }
        }
        #endregion
        #endregion
    }
}
